import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.tree.*;

public class FileTree extends JFrame implements MouseListener, TreeSelectionListener {
	DefaultMutableTreeNode node;
	File dir;
	String fileName=null,nextLine=null;
	public FileTree(File runDir) {
		super("Project Directory");
		dir=runDir;
		setForeground(Color.black);
		setBackground(Color.lightGray);
		JTree tree = new JTree(addNodes(null, dir));
		tree.addTreeSelectionListener(this);
		tree.addMouseListener(this);
		JScrollPane scrollpane = new JScrollPane();
		scrollpane.getViewport().add(tree);
		getContentPane().add(scrollpane,BorderLayout.CENTER);
		setSize(640,480);
		setVisible(true);
		setResizable(false);
	}
	public void valueChanged(TreeSelectionEvent e) {
		node = (DefaultMutableTreeNode) e.getPath().getLastPathComponent();
	}
	public void mousePressed(MouseEvent me) {
	}
	public void mouseReleased(MouseEvent me) {}
	public void mouseEntered(MouseEvent me) {}
	public void mouseExited(MouseEvent me) {}
	public void mouseClicked(MouseEvent me) {
		int noc=me.getClickCount();
		if (noc==2) {
			openRelevantApplication(node);
		}
	}

	public void openRelevantApplication(DefaultMutableTreeNode node) {
		fileName=node.toString();
		if (fileName.endsWith(".pdb") || fileName.endsWith(".PDB")) {
			prepareExpectFile("rasmol");
			try {
				String[] runcomm={"/bin/bash","-c","expect /tmp/ras.exp"};
				Process proc=Runtime.getRuntime().exec(runcomm);
			}
			catch (IOException ioe) {}
		}
		if (fileName.endsWith(".xvg") || fileName.endsWith(".XVG")) {
			try {
				String[] runcomm={"/bin/bash","-c","xmgrace "+dir.toString()+"/"+fileName};
				Process proc=Runtime.getRuntime().exec(runcomm);
				BufferedReader br=new BufferedReader(new InputStreamReader(proc.getErrorStream()));
				while((nextLine=br.readLine())!=null) {
					if(nextLine.indexOf("command")!=-1) {
						JOptionPane.showMessageDialog(this,"Could not open the file with the application assigned to it\nPlease make sure xmgrace is installed and try again","Fatal Error",JOptionPane.ERROR_MESSAGE);
					}
				}
			}
			catch (IOException ioe) {}
		}
		if (fileName.indexOf(".top")!=-1 || fileName.indexOf(".gro")!=-1 || fileName.indexOf(".itp")!=-1 || fileName.indexOf(".mdp")!=-1 || fileName.indexOf(".log")!=-1) {
			JFrame f=new JFrame(fileName);
			f.setLocationRelativeTo(this);
			JTextArea jta=new JTextArea(5,5);
			jta.setEditable(false);
			jta.setMargin(new Insets(10,10,10,10));
			jta.setFont(new Font("monospaced",Font.PLAIN,14));
			jta.setCaretPosition(0);
			f.getContentPane().add(new JScrollPane(jta));
			try {
				BufferedReader br=new BufferedReader(new FileReader(dir.toString()+"/"+fileName));
				jta.read(br,null);
			}
			catch (FileNotFoundException fnfe) {
				try {
					BufferedReader br=new BufferedReader(new FileReader(dir.toString()+"/logs/"+fileName));
					jta.read(br,null);
				}
				catch (FileNotFoundException fnf) {}
				catch (IOException io) {}
			}
			catch (IOException ioe) {}
			f.setSize(640,480);
			f.setVisible(true);
		}
		if (fileName.indexOf(".tpr")!=-1 || fileName.indexOf(".trr")!=-1 || fileName.indexOf(".edr")!=-1) {
			JOptionPane.showMessageDialog(this,"This file cannot be opened as such\nPlease run gmxdump before attempting to open","Fatal Error",JOptionPane.ERROR_MESSAGE);
		}
	}
	public void prepareExpectFile(String appln) {
		try {
			BufferedWriter bw=new BufferedWriter(new FileWriter("/tmp/ras.exp"));
			bw.write("#!/bin/expect\n");
			if (appln.equals("rasmol")) {
				bw.write("spawn -noecho rasmol\n");
				bw.write("expect \"RasMol> \"\n");
				bw.write("send \"load "+dir.toString()+"/"+fileName+"\\r\"\n");
				bw.write("set timeout -1\n");
				bw.write("expect EOF\n");
				bw.close();
			}
		}
		catch(IOException ioe) {}
	}

	DefaultMutableTreeNode addNodes(DefaultMutableTreeNode curTop, File dir) {
		String curPath = dir.getPath();
		DefaultMutableTreeNode curDir = new DefaultMutableTreeNode(curPath);
		if (curTop != null) {
			curTop.add(curDir);
		}
		Vector ol = new Vector();
		String[] tmp = dir.list();
		for (int i = 0; i < tmp.length; i++)
			ol.addElement(tmp[i]);
		Collections.sort(ol, String.CASE_INSENSITIVE_ORDER);
		File f;
		Vector files = new Vector();
		for (int i = 0; i < ol.size(); i++) {
			String thisObject = (String) ol.elementAt(i);
			String newPath;
			if (curPath.equals("."))
				newPath = thisObject;
			else
				newPath = curPath + File.separator + thisObject;
			if ((f = new File(newPath)).isDirectory())
				addNodes(curDir, f);
			else
				files.addElement(thisObject);
		}
		for (int fnum = 0; fnum < files.size(); fnum++)
			curDir.add(new DefaultMutableTreeNode(files.elementAt(fnum)));
		return curDir;
	}
	
	public Dimension getMinimumSize() {
		return new Dimension(200, 400);
	}
	
	public Dimension getPreferredSize() {
		return new Dimension(200, 400);
	}
	public static void main(String[] av) {
	}
}
