import java.io.*;
import java.util.*;
import java.lang.Math.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

public class automacs extends JFrame implements ActionListener
{
	FileTree ft;
	JLabel lTitle=new JLabel("<html>GUIMACS Automaton</html>");
	String info=new String("<html><body><div style=\"width:800; font-family:sans-serif; text-align:justify; padding:20px;\">Molecular Dynamics simulations are made easy by the GUIMACS Automaton. It is a powerful program capable of running a complete simulation from scratch.All it requires is a PDB file and the length of simulation in picoseconds as input. In case an alternate solvent is to be used (opposed to water), the corresponding .gro file needs to be input and the Automaton does the rest.<br><br>Output files will be written to the directory you had selected prior to starting the Automaton. For understanding, the output files will be tagged with two letter codes depicting the step during simulation they belong to. The two letter codes for the standard set of programs needed to run a complete simulation are listed below.</div><table border=0 align=center cellspacing=10><tr><td>pdb2gmx</td><td>pg</td></tr><tr><td>editconf</td><td>ec</td></tr><tr><td>genbox</td><td>gb</td></tr><tr><td>grompp</td><td>gp</td></tr><tr><td>genion</td><td>gi</td></tr><tr><td>mdrun</td><td>md</td></tr></table></body></html>");
	JLabel lInfo=new JLabel(info);
	Font f=new Font("helvetica",Font.BOLD,20);
	JScrollPane jsp;
	JPanel main,cPanel;
	JTextField tPdb,tTime,tEmdp,tFmdp;
	JButton bPdb,bProceed,bEmdp,bFmdp;
	JRadioButton rTime,rMdp;
	JFileChooser pdbjfc,mdpjfc;
	JWindow win;
	File pdb,mdp;
	JComboBox jcb;
	String runDir=null,nextLine=null,pdbfile=null,emdpfile=null,fmdpfile=null,forcefield=null,runstr=null,pdbfname=null,mdpfname=null,pdbid=null,charge=null,ionname=null;
	Vector ffdat=new Vector();
	Hashtable residues=new Hashtable();
	int i=0,flag=0,evl=-1,pfstat=-1,psstat=-1,ions=0,gistat=-1,time=10,timein=0,mdpin=1;
	Thread runner;
	String[] runcomm = new String[3];
	automacs(Container content, String runDir)
	{
		this.runDir=runDir;
		(new File(runDir+"/logs")).mkdir();
		pdbjfc=new JFileChooser();
		pdbjfc.setCurrentDirectory(new java.io.File(runDir));
		mdpjfc=new JFileChooser();
		mdpjfc.setCurrentDirectory(new java.io.File(runDir));
		runcomm[0]="/bin/sh";
		runcomm[1]="-c";
		win=new JWindow();
		populateFrame(content);
	}
	public void actionPerformed(ActionEvent ae)
	{
		String command=ae.getActionCommand();
		if(command.equals("time")) {
			timein=0;
			mdpin=1;
			tTime.setEnabled(true);
			tFmdp.setEnabled(false);
			bFmdp.setEnabled(false);
		}
		if(command.equals("mdp")) {
			timein=1;
			mdpin=0;
			tTime.setEnabled(false);
			tFmdp.setEnabled(true);
			bFmdp.setEnabled(true);
		}
		if(command.equals("pbrowse")) {
			pdbjfc.setDialogTitle("Choose input PDB file");
			pdbjfc.addChoosableFileFilter(new PDBFilter());
			int retval=pdbjfc.showOpenDialog(this);
			if(retval==JFileChooser.APPROVE_OPTION) {
				pdb=pdbjfc.getSelectedFile();
				tPdb.setText(pdb.toString());
			}
		}
		if(command.equals("mbrowse") || command.equals("fbrowse")) {
			mdpjfc.setDialogTitle("Choose MDP file");
			mdpjfc.addChoosableFileFilter(new MDPFilter());
			int returnval=mdpjfc.showOpenDialog(this);
			if(returnval==JFileChooser.APPROVE_OPTION) {
				mdp=mdpjfc.getSelectedFile();
				if(command.equals("mbrowse")) {
					tEmdp.setText(mdp.toString());
				}
				if(command.equals("fbrowse")) {
					tFmdp.setText(mdp.toString());
				}
			}
		}
		if(command.equals("proceed")) {
			validateInput(command);
		}
	}
	public void populateFrame(Container content)
	{
		main=new JPanel();
		jsp=new JScrollPane();
		jsp.getViewport().add(main);
		jsp.setAutoscrolls(true);
		
		content.add(jsp,BorderLayout.CENTER);

		GridBagLayout gb=new GridBagLayout();
		GridBagConstraints gc=new GridBagConstraints();
		main.setLayout(gb);
				
		lTitle.setFont(f);
		gc.gridx=0;
		gc.gridy=0;
		gc.weightx=0.3;
		gc.weighty=0.3;
		gb.setConstraints(lTitle,gc);
		main.add(lTitle);

		gc.gridx=0;
		gc.gridy=1;
		gb.setConstraints(lInfo,gc);
		lInfo.setText(info);
		main.add(lInfo);
		
		cPanel=new JPanel();
		gc.gridx=0;
		gc.gridy=2;
		gb.setConstraints(cPanel,gc);
		main.add(cPanel);

		cPanel.setBorder(BorderFactory.createTitledBorder("Input Parameters [ * denotes optional parameters ]"));
		SpringLayout opt=new SpringLayout();
		cPanel.setLayout(opt);
		JLabel lPdb=new JLabel("My input PDB filename :",JLabel.CENTER);
		tPdb=new JTextField(15);
		bPdb=new JButton("Browse");
		bPdb.addActionListener(this);
		bPdb.setActionCommand("pbrowse");
		lPdb.setLabelFor(tPdb);
		cPanel.add(lPdb);
		cPanel.add(tPdb);
		cPanel.add(bPdb);
		cPanel.setPreferredSize(new Dimension(750,350));
		opt.putConstraint(SpringLayout.WEST,lPdb,20,SpringLayout.WEST,cPanel);
		opt.putConstraint(SpringLayout.NORTH,lPdb,20,SpringLayout.NORTH,cPanel);
		opt.putConstraint(SpringLayout.WEST,tPdb,20,SpringLayout.EAST,lPdb);
		opt.putConstraint(SpringLayout.NORTH,tPdb,20,SpringLayout.NORTH,cPanel);
		opt.putConstraint(SpringLayout.WEST,bPdb,10,SpringLayout.EAST,tPdb);
		opt.putConstraint(SpringLayout.NORTH,bPdb,20,SpringLayout.NORTH,cPanel);

		JLabel lFfield=new JLabel("My force field :",JLabel.CENTER);
		try{
			BufferedReader br=new BufferedReader(new FileReader(System.getProperty("GMXLIB")+"/FF.dat"));
			while((nextLine=br.readLine())!=null)
			{
				if(nextLine.length()>5)
				{
					ffdat.addElement(nextLine.trim());
				}
			}
		}
		catch(FileNotFoundException fnfe){}
		catch(IOException ioe){}
		jcb=new JComboBox(ffdat);
		cPanel.add(lFfield);
		cPanel.add(jcb);
		opt.putConstraint(SpringLayout.NORTH,lFfield,30,SpringLayout.SOUTH,lPdb);
		opt.putConstraint(SpringLayout.WEST,lFfield,70,SpringLayout.WEST,cPanel);
		opt.putConstraint(SpringLayout.NORTH,jcb,20,SpringLayout.SOUTH,tPdb);
		opt.putConstraint(SpringLayout.WEST,jcb,20,SpringLayout.EAST,lFfield);
		
		JLabel lType=new JLabel("I wish to specify :",JLabel.CENTER);
		rTime=new JRadioButton("Length of simulation");
		rMdp=new JRadioButton("Custom MDP file");
		cPanel.add(lType);
		cPanel.add(rTime);
		rTime.addActionListener(this);
		rTime.setActionCommand("time");
		rTime.setSelected(true);
		cPanel.add(rMdp);
		rMdp.addActionListener(this);
		rMdp.setActionCommand("mdp");
		opt.putConstraint(SpringLayout.NORTH,lType,20,SpringLayout.SOUTH,lFfield);
		opt.putConstraint(SpringLayout.WEST,lType,55,SpringLayout.WEST,cPanel);
		opt.putConstraint(SpringLayout.NORTH,rTime,20,SpringLayout.SOUTH,jcb);
		opt.putConstraint(SpringLayout.WEST,rTime,20,SpringLayout.EAST,lType);
		opt.putConstraint(SpringLayout.NORTH,rMdp,20,SpringLayout.SOUTH,jcb);
		opt.putConstraint(SpringLayout.WEST,rMdp,10,SpringLayout.EAST,rTime);
		
		JLabel lTime=new JLabel("<html>Length of simulation :<br>(in ps)</html>",JLabel.CENTER);
		tTime=new JTextField(10);
		JLabel lTinfo=new JLabel("(1 ns = 1000 ps)");
		bProceed=new JButton("Proceed");
		bProceed.addActionListener(this);
		bProceed.setActionCommand("proceed");
		cPanel.add(lTime);
		cPanel.add(tTime);
		cPanel.add(lTinfo);
		cPanel.add(bProceed);
		opt.putConstraint(SpringLayout.NORTH,lTime,20,SpringLayout.SOUTH,lType);
		opt.putConstraint(SpringLayout.WEST,lTime,25,SpringLayout.WEST,cPanel);
		opt.putConstraint(SpringLayout.NORTH,tTime,20,SpringLayout.SOUTH,rTime);
		opt.putConstraint(SpringLayout.WEST,tTime,25,SpringLayout.EAST,lTime);
		opt.putConstraint(SpringLayout.WEST,lTinfo,10,SpringLayout.EAST,tTime);
		opt.putConstraint(SpringLayout.NORTH,lTinfo,20,SpringLayout.SOUTH,rMdp);
		
		JLabel lEmdp=new JLabel("<html>*Input mdp filename :<br>for energy minimization</html>",JLabel.CENTER);
		tEmdp=new JTextField(15);
		bEmdp=new JButton("Browse");
		bEmdp.addActionListener(this);
		bEmdp.setActionCommand("mbrowse");
		cPanel.add(lEmdp);
		cPanel.add(tEmdp);
		cPanel.add(bEmdp);
		opt.putConstraint(SpringLayout.NORTH,lEmdp,25,SpringLayout.SOUTH,lTime);
		opt.putConstraint(SpringLayout.WEST,lEmdp,25,SpringLayout.WEST,cPanel);
		opt.putConstraint(SpringLayout.NORTH,tEmdp,20,SpringLayout.SOUTH,tTime);
		opt.putConstraint(SpringLayout.WEST,tEmdp,10,SpringLayout.EAST,lEmdp);
		opt.putConstraint(SpringLayout.NORTH,bEmdp,20,SpringLayout.SOUTH,lTinfo);
		opt.putConstraint(SpringLayout.WEST,bEmdp,10,SpringLayout.EAST,tEmdp);
		
		JLabel lFmdp=new JLabel("<html>*Input mdp filename :<br>for molecular dynamics</html>",JLabel.CENTER);
		tFmdp=new JTextField(15);
		bFmdp=new JButton("Browse");
		bFmdp.addActionListener(this);
		bFmdp.setActionCommand("fbrowse");
		cPanel.add(lFmdp);
		cPanel.add(tFmdp);
		tFmdp.setEnabled(false);
		cPanel.add(bFmdp);
		bFmdp.setEnabled(false);
		opt.putConstraint(SpringLayout.NORTH,lFmdp,25,SpringLayout.SOUTH,lEmdp);
		opt.putConstraint(SpringLayout.WEST,lFmdp,25,SpringLayout.WEST,cPanel);
		opt.putConstraint(SpringLayout.NORTH,tFmdp,30,SpringLayout.SOUTH,tEmdp);
		opt.putConstraint(SpringLayout.WEST,tFmdp,15,SpringLayout.EAST,lFmdp);
		opt.putConstraint(SpringLayout.NORTH,bFmdp,25,SpringLayout.SOUTH,bEmdp);
		opt.putConstraint(SpringLayout.WEST,bFmdp,10,SpringLayout.EAST,tFmdp);
		
		opt.putConstraint(SpringLayout.WEST,bProceed,10,SpringLayout.EAST,lFmdp);
		opt.putConstraint(SpringLayout.NORTH,bProceed,20,SpringLayout.SOUTH,tFmdp);
		
		ButtonGroup group=new ButtonGroup();
		group.add(rTime);
		group.add(rMdp);
	}
	public void validateInput(String command)
	{
		if(command.equals("proceed")) {
			if(tPdb.getText().trim().length()==0 || (timein==0 && tTime.getText().trim().length()==0) || (mdpin==0 && tFmdp.getText().trim().length()==0)) {
				if(tPdb.getText().trim().length()==0) {
					JOptionPane.showMessageDialog(this,"Please choose a valid pdb file to continue","Invalid pdb file",JOptionPane.ERROR_MESSAGE);
					tPdb.requestFocus();
				}
				if(timein==0 && tTime.getText().trim().length()==0) {
					JOptionPane.showMessageDialog(this,"Please specify the length of simulation","Invalid time",JOptionPane.ERROR_MESSAGE);
					tTime.requestFocus();
				}
				if(mdpin==0 && tFmdp.getText().trim().length()==0) {
					JOptionPane.showMessageDialog(this,"Please specify a valid mdp file","Invalid MDP entry",JOptionPane.ERROR_MESSAGE);
					tFmdp.requestFocus();
				}
			}
			else {
				pdbfile=tPdb.getText().trim();
				String[] fields=jcb.getSelectedItem().toString().split(" ");
				forcefield=fields[0].trim().substring(2,fields[0].length());
				String[] dirs=pdbfile.split("/");
				pdbfname=dirs[dirs.length-1];
				pdbid=pdbfname.substring(0,pdbfname.length()-4);
				time=Integer.parseInt(tTime.getText().trim());
				if(tEmdp.getText().trim().length()==0) {
					emdpfile=System.getProperty("TXTPATH")+"/lists/em.mdp";
				}
				else {
					emdpfile=tEmdp.getText().trim();
				}
				if(mdpin==0) {
					fmdpfile=tFmdp.getText().trim();
				}
				else {
					fmdpfile=System.getProperty("TXTPATH")+"/lists/full.mdp";
				}
				try{
					BufferedReader br=new BufferedReader(new FileReader(pdbfile));
					BufferedWriter bw=new BufferedWriter(new FileWriter(new File(runDir+"/"+pdbid+"_in.pdb")));
					BufferedReader fr=new BufferedReader(new FileReader(System.getProperty("GMXLIB")+"/aminoacids.dat"));
					while((nextLine=fr.readLine())!=null)
					{
						//String[] res=nextLine.split("=");
						residues.put(nextLine.trim(),nextLine.trim());
					}
					while((nextLine=br.readLine())!=null)
					{
						if(nextLine.startsWith("ATOM") && residues.contains(nextLine.substring(17,20).trim()))
						{
							bw.write(nextLine+"\n");
						}
					}
					bw.close();
					if(mdpin==1) {
						Vector mdpvec=new Vector();
						BufferedReader mr=new BufferedReader(new FileReader(fmdpfile));
						while((nextLine=mr.readLine())!=null) {
							mdpvec.addElement(nextLine);
						}
						fmdpfile=runDir+"/full.mdp";
						BufferedWriter mw=new BufferedWriter(new FileWriter(fmdpfile));
						for(Enumeration e=mdpvec.elements();e.hasMoreElements();) {
							String ne=e.nextElement().toString();
							if(ne.startsWith("nsteps")) {
								String[] vals=ne.split("=");
								int ns=time*500;
								vals[1]="  "+ns;
								mw.write(vals[0]+"="+vals[1]+"\n");							
							}
							else {
								mw.write(ne+"\n");
							}
						}
						mw.close();
					}
					runAutomaton();
				}
				catch(FileNotFoundException fnfe)
				{
					JOptionPane.showMessageDialog(this,fnfe,"File Not Found !",JOptionPane.ERROR_MESSAGE);
				}
				catch(IOException ioe)
				{
					JOptionPane.showMessageDialog(this,"IO Exception\n"+ioe,"IO Error !",JOptionPane.ERROR_MESSAGE);
				}
			}
		}
	}
	public void runAutomaton()
	{
		runner=new Thread()	{
			public void run() {
				try{
					JLabel lStatus=new JLabel("Started Automaton...",JLabel.CENTER);
					win.getContentPane().add(lStatus);
					win.setLocation(100,100);
					lStatus.setFont(new Font("serif",Font.BOLD,30));
					lStatus.setBackground(new Color(167,10,158));
					lStatus.setForeground(Color.WHITE);
					lStatus.setOpaque(true);
					lStatus.setBorder(BorderFactory.createCompoundBorder(BorderFactory.createLineBorder(Color.WHITE,2),BorderFactory.createEmptyBorder(10,10,10,10)));
					win.setSize(600,100);
					win.show();
					pfstat=prepareFile("pgmx");
					if(pfstat==0){
						Process procpgmx=Runtime.getRuntime().exec(runcomm,null,(new File(runDir)));
						lStatus.setText("Processing input PDB...");
						int evlpgmx=procpgmx.waitFor();
					}
					else{
						win.hide();
						JOptionPane.showMessageDialog(null,"You do not seem to have permissions to write to your /tmp directory.\nPlease make sure you have enough permissions before you proceed","Fatal Error",JOptionPane.ERROR_MESSAGE);
						return;
					}
					psstat=proceedStatus("pgmx");
					if(psstat==0) {
						lStatus.setText("Processed PDB file...");
						pfstat=prepareFile("ecnf");
						Process procecnf=Runtime.getRuntime().exec(runcomm,null,(new File(runDir)));
						lStatus.setText("Generating box...");
						int evlecnf=procecnf.waitFor();
						psstat=proceedStatus("ecnf");
						if(psstat==0) {
							lStatus.setText("Finished generating box...");
							pfstat=prepareFile("gbox");
							Process procgbox=Runtime.getRuntime().exec(runcomm,null,(new File(runDir)));
							lStatus.setText("Solvating the box...");
							int evlgbox=procgbox.waitFor();
							psstat=proceedStatus("gbox");
							if(psstat==0) {
								lStatus.setText("Finished solvating the box...");
								pfstat=prepareFile("gpem");
								Process procgpem=Runtime.getRuntime().exec(runcomm,null,(new File(runDir)));
								lStatus.setText("Running GROMACS Preprocessor...");
								int evlgpem=procgpem.waitFor();
								psstat=proceedStatus("gpem");
								if(psstat==0) {
									lStatus.setText("Finished preprocessing...");
									if(charge.length()!=0) {
										ions=Integer.parseInt(charge.trim());
										gistat=0;
										if(ions>0) {
											if(forcefield.equals("gmx") || forcefield.equals("gmx2")) {
												ionname="Cl";
											}
											else {
												ionname="CL-";
											}
										}
										else if(ions<0) {
											if(forcefield.equals("gmx") || forcefield.equals("gmx2")) {
												ionname="Na";
											}
											else {
												ionname="NA+";
											}
										}
										else{
											lStatus.setText("System Neutral...");
											gistat=1;
										}
										if(gistat==0) {
											pfstat=prepareFile("gion");
											Process procgion=Runtime.getRuntime().exec(runcomm,null,(new File(runDir)));
											lStatus.setText("Neutralizing the system...");
											int evlgion=procgion.waitFor();
											psstat=proceedStatus("gion");
											if(psstat==0) {
												lStatus.setText("Finished neutralizing the system...");
												Vector mdpvec=new Vector();
												BufferedReader mr=new BufferedReader(new FileReader(fmdpfile));
												while((nextLine=mr.readLine())!=null) {
													mdpvec.addElement(nextLine);
												}
												lStatus.setText("Generating MDP file...");
												BufferedWriter mw=new BufferedWriter(new FileWriter(fmdpfile));
												for(Enumeration e=mdpvec.elements();e.hasMoreElements();) {
													String ne=e.nextElement().toString();
													if(ne.startsWith("tc-grps")) {
														String[] tcg=ne.split("=");
														tcg[1]+="\t"+ionname;
														mw.write(tcg[0]+"="+tcg[1]+"\n");
													}
													else if(ne.startsWith("tau_t")) {
														String[] tt=ne.split("=");
														tt[1]+="\t0.1";
														mw.write(tt[0]+"="+tt[1]+"\n");
													}
													else if(ne.startsWith("ref_t")) {
														String[] rt=ne.split("=");
														rt[1]+="\t300";
														mw.write(rt[0]+"="+rt[1]+"\n");
													}
													else if(ne.startsWith("energygrps")) {
														String[] eg=ne.split("=");
														eg[1]+="\t"+ionname;
														mw.write(eg[0]+"="+eg[1]+"\n");
													}
													else {
														mw.write(ne+"\n");
													}
												}
												mw.close();
												lStatus.setText("Finished generating MDP file...");
												pfstat=prepareFile("emio");
												Process procemio=Runtime.getRuntime().exec(runcomm,null,(new File(runDir)));
												lStatus.setText("Generating run input file...");
												int evlemio=procemio.waitFor();
												psstat=proceedStatus("emio");
												if(psstat==0) {
													lStatus.setText("Finished generating run input...");
												}
												else {
													win.hide();
													JOptionPane.showMessageDialog(null,"Automaton failed to run grompp.\nThis means that the run input file for energy minimization could not be created.\nPlease refer to log file for more details","Fatal Error",JOptionPane.ERROR_MESSAGE);
													return;
												}
											}
											else {
												win.hide();
												JOptionPane.showMessageDialog(null,"Automaton failed to run genion.\nThis means that the system is not neutralized.\nRunning simulation in this stage might render the system unstable.\nPlease refer to log file for details.\nTry to fix the problem and proceed.","Fatal Error",JOptionPane.ERROR_MESSAGE);
												return;
											}
										}
									}
									pfstat=prepareFile("gpmd");
									Process procgpmd=Runtime.getRuntime().exec(runcomm,null,(new File(runDir)));
									lStatus.setText("Running Energy Minimization...");
									int evlgpmd=procgpmd.waitFor();
									psstat=proceedStatus("gpmd");
									if(psstat==0) {
										lStatus.setText("Finished Energy Minimization...");
										pfstat=prepareFile("mdem");
										Process procmdem=Runtime.getRuntime().exec(runcomm,null,(new File(runDir)));
										lStatus.setText("Generating mdrun input file...");
										int evlmdem=procmdem.waitFor();
										psstat=proceedStatus("mdem");
										if(psstat==0) {
											lStatus.setText("Finished generating input file...");
											pfstat=prepareFile("mrun");
											Process procmrun=Runtime.getRuntime().exec(runcomm,null,(new File(runDir)));
											lStatus.setText("Running Simulation...");
											int evlmrun=procmrun.waitFor();
											psstat=proceedStatus("mrun");
											if(psstat==0) {
												pfstat=prepareFile("grms");
												Process procgrms=Runtime.getRuntime().exec(runcomm,null,(new File(runDir)));
												lStatus.setText("Calculating RMSD...");
												int evlgrms=procgrms.waitFor();
												psstat=proceedStatus("grms");										
												if(psstat==0) {
													generateImage("rms");
													pfstat=prepareFile("gcnf");
													Process procgcnf=Runtime.getRuntime().exec(runcomm,null,(new File(runDir)));
													lStatus.setText("Superimposing structures...");
													int evlgcnf=procgcnf.waitFor();
													psstat=proceedStatus("gcnf");
													if(psstat==0) {
														win.hide();
														JOptionPane.showMessageDialog(null,"Automaton finished running the simulation.\nPlease have a look at the log files for further details.\n\nThank you for using AutoMACS","Done",JOptionPane.INFORMATION_MESSAGE);
														ft=new FileTree(new java.io.File(runDir));														
													}													
													else {
														win.hide();
														JOptionPane.showMessageDialog(null,"There was a problem superimposing structures.\nPlease refer to log file for details","Done",JOptionPane.ERROR_MESSAGE);
													}	
												}
											}
											else {
												win.hide();
												JOptionPane.showMessageDialog(null,"There was a problem calculating RMSD.\nPlease refer to log file for details","Done",JOptionPane.ERROR_MESSAGE);
											}
										}
										else {
											win.hide();
											JOptionPane.showMessageDialog(null,"Automaton failed to run grompp.\nThis means that the run input file for molecular dynamics could not be created.\nPlease refer to log file for further details","Fatal Error",JOptionPane.ERROR_MESSAGE);
											return;
										}
									}
									else {
										win.hide();
										JOptionPane.showMessageDialog(null,"Automaton failed to run energy minimization.\nPlease refer to log file for details","Fatal Error",JOptionPane.ERROR_MESSAGE);
										return;
									}									
								}
								else {
									win.hide();
									JOptionPane.showMessageDialog(null,"Automaton failed to run grompp.\nThis means that the run input file for energy minimization could not be created.\nPlease refer to log file for further details.","Fatal Error",JOptionPane.ERROR_MESSAGE);
									return;
								}
							}
							else {
								win.hide();
								JOptionPane.showMessageDialog(null,"Automaton failed to run genbox. Please refer to log file for details.\nTry to fix the problem and proceed.","Fatal Error",JOptionPane.ERROR_MESSAGE);
								return;
							}
						}
						else {
							win.hide();
							JOptionPane.showMessageDialog(null,"Automaton failed to run editconf. Please refer to log file for details.\nTry to fix the problem and proceed.","Fatal Error",JOptionPane.ERROR_MESSAGE);
							return;
						}
					}
					else {
						win.hide();
						JOptionPane.showMessageDialog(null,"Automaton failed to run pdb2gmx successfully. This is the first step in a molecular dynamics simulation.\nThis could mean that the input PDB file is not complete.\n\nPlease make sure the PDB file depicts the complete structure of the protein you wish to simulate.\n\nHint: WHAT IF web interface might be a good idea to fix the problem.","Fatal Error",JOptionPane.ERROR_MESSAGE);
						return;
					}
				}
				catch(FileNotFoundException fnfe) {
					JOptionPane.showMessageDialog(null,fnfe,"File Not Found !",JOptionPane.ERROR_MESSAGE);
				}
				catch(IOException ioe) {
					JOptionPane.showMessageDialog(null,ioe,"IO Exception !",JOptionPane.ERROR_MESSAGE);
				}
				catch(InterruptedException inte) {
					JOptionPane.showMessageDialog(null,inte,"Interrupted Exception !",JOptionPane.ERROR_MESSAGE);
				}
			}
			public int prepareFile(final String code) {
				pfstat=0;
				if(code.equals("pgmx")) {
					runstr="pdb2gmx -ff "+forcefield+" -f "+runDir+"/"+pdbid+"_in.pdb"+" -o "+runDir+"/"+pdbid+"pg -p "+runDir+"/"+pdbid+"pg";
				}
				if(code.equals("ecnf")) {
					runstr="editconf -f "+pdbid+"pg -o "+pdbid+"ec -c -d 0.8";
				}
				if(code.equals("gbox")) {
					runstr="genbox -cp "+pdbid+"ec -cs -o "+pdbid+"gb -p "+pdbid+"pg";
				}
				if(code.equals("gion")) {
					if(ionname.equals("Na") || ionname.equals("NA+")) {
						runstr="genion -s "+pdbid+"em -o "+pdbid+"gi -np "+Math.abs(ions)+" -pname "+ionname;
					}
					if(ionname.equals("Cl") || ionname.equals("CL-")) {
						runstr="genion -s "+pdbid+"em -o "+pdbid+"gi -nn "+ions+" -nname "+ionname;
					}
				}
				if(code.equals("gpem")) {
					if(gistat==0) {
						runstr="grompp -f "+emdpfile+" -c "+pdbid+"gi -p "+pdbid+"pg -o "+pdbid+"em";
					}
					else {
						runstr="grompp -f "+emdpfile+" -c "+pdbid+"gb -p "+pdbid+"pg -o "+pdbid+"em";
					}
				}
				if(code.equals("emio")) {
					runstr="grompp -f "+emdpfile+" -c "+pdbid+"gi -p "+pdbid+"gi -o "+pdbid+"em";
				}
				if(code.equals("gpmd")) {
					runstr="mdrun -v -s "+pdbid+"em -o "+pdbid+"em -c "+pdbid+"em -e "+pdbid+"em";
				}
				if(code.equals("mdem")) {
					if(gistat==0) {
						runstr="grompp -f "+fmdpfile+" -c "+pdbid+"gi -p "+pdbid+"gi -o "+pdbid+"md";
					}
					else {
						runstr="grompp -f "+fmdpfile+" -c "+pdbid+"gb -p "+pdbid+"pg -o "+pdbid+"md";
					}
				}
				if(code.equals("mrun")) {
					runstr="mdrun -v -s "+pdbid+"md -o "+pdbid+"md -c "+pdbid+"md -e "+pdbid+"md";
				}
				if(code.equals("grms")) {
					runstr="g_rms -s "+pdbid+"md -f "+pdbid+"md -o "+pdbid+"_rms";
				}
				if(code.equals("gcnf")) {
					runstr="g_confrms -f1 "+pdbid+"pg.gro -f2 "+pdbid+"md.gro -o "+pdbid+"_fit.pdb";
				}
				try {
					BufferedWriter bw=new BufferedWriter(new FileWriter("/tmp/"+code+".exp"));
					bw.write("#!/bin/expect\n");
					bw.write("spawn -noecho "+runstr+"\n");
					if(code.equals("gion")) {
						bw.write("expect \"Select a group:\"\n");
						bw.write("send \"12\\r\"\n");
					}
					if(code.equals("grms") || code.equals("gcnf")) {
						bw.write("expect \"Select a group:\"\n");
						bw.write("send \"4\\r\"\n");
						bw.write("expect \"Select a group:\"\n");
						bw.write("send \"4\\r\"\n");
					}
					bw.write("set timeout -1\n");
					bw.write("expect EOF\n");
					bw.close();
					runcomm[2]="expect /tmp/"+code+".exp > logs/"+code+".log";
				}
				catch(IOException ioe){
					pfstat=1;
				}
				return pfstat;
			}
			public int proceedStatus(String code) {
				psstat=0;
				try {
					BufferedReader br=new BufferedReader(new FileReader(runDir+"/logs/"+code+".log"));
					while((nextLine=br.readLine())!=null) {
						if(nextLine.indexOf("error:")!=-1) {
							psstat=1;
							break;
						}
						if(nextLine.startsWith("Total charge")) {
							String[] fields=nextLine.split("\\.");
							String[] flds=fields[0].split(" ");
							charge=flds[flds.length-1].trim();
						}	
					}
					if(code.equals("gion")) {
						Vector toplines=new Vector();
						BufferedReader gr=new BufferedReader(new FileReader(runDir+"/"+pdbid+"pg.top"));
						while((nextLine=gr.readLine())!=null) {
							toplines.addElement(nextLine);
						}
						String[] fields=toplines.lastElement().toString().split(" ");
						int sol=Integer.parseInt(fields[fields.length-1])-Math.abs(ions);
						String newLine="SOL";
						for(int i=0;i<18-Integer.toString(sol).length();i++) {
							newLine+=" ";
						}
						newLine+=Integer.toString(sol);
						toplines.set(toplines.size()-1,newLine);
						BufferedWriter gw=new BufferedWriter(new FileWriter(runDir+"/"+pdbid+"gi.top"));
						for(Enumeration e=toplines.elements();e.hasMoreElements();) {
							String nextLine=(String)e.nextElement();
							gw.write(nextLine+"\n");
						}
						String lastLine=ionname;
						for(int j=0;j<19-Integer.toString(Math.abs(ions)).length();j++) {
							lastLine+=" ";
						}
						lastLine+=Integer.toString(Math.abs(ions));
						gw.write(lastLine+"\n");
						gw.close();
					}
						
				}
				catch(FileNotFoundException fnfe) {
					System.out.println(fnfe);
					psstat=1;
				}
				catch(IOException ioe) {
					System.out.println(ioe);
					psstat=1;
				}
				return psstat;
			}
			public void generateImage(String code) {
				try {
					BufferedWriter bw=new BufferedWriter(new FileWriter("/tmp/"+code+".exp"));			
					runcomm[2]="xmgrace -hdevice JPEG -hardcopy "+runDir+"/"+pdbid+"_"+code+".xvg -printfile "+runDir+"/"+pdbid+"_"+code+".jpg";
					Process imgGen=Runtime.getRuntime().exec(runcomm,null,(new File(runDir)));
					int retval=imgGen.waitFor();
				}
				catch(IOException ioe) {
					JOptionPane.showMessageDialog(null,"There was a problem generating the jpg image.\nPlease make sure the JPEG library exists on your computer","Fatal Error",JOptionPane.ERROR_MESSAGE);
				}
				catch(InterruptedException inte) {}
			}
	};
		runner.start();
		
	}
	class PDBFilter extends javax.swing.filechooser.FileFilter {
		public boolean accept(File f) {
			return f.isDirectory() || f.getName().toLowerCase().endsWith(".pdb");
		}
		public String getDescription() {
			return "*.pdb (Protein Data Bank files)";
		}
	}
	class MDPFilter extends javax.swing.filechooser.FileFilter {
		public boolean accept(File f) {
			return f.isDirectory() || f.getName().toLowerCase().endsWith(".mdp");
		}
		public String getDescription() {
			return "*.mdp (MD Parameter files)";
		}
	}
}
