package vault;
import java.io.*;
import java.awt.*;
import javax.swing.*;
import java.awt.event.*;
import javax.swing.event.*;

public class g_helix implements ActionListener
{
	String title="g_helix";
	String help="<html>g_helix computes all kind of helix properties. First, the peptide is checked<br>to find the longest helical part. This is determined by Hydrogen bonds and<br>Phi/Psi angles. That bit is fitted to an ideal helix around the Z-axis and<br>centered around the origin. Then the following properties are computed:<br><br>1. Helix radius (file radius.xvg). This is merely the RMS deviation in two<br>dimensions for all Calpha atoms. it is calced as sqrt((SUM<br>i(x^2(i)+y^2(i)))/N), where N is the number of backbone atoms. For an ideal<br>helix the radius is 0.23 nm<br>2. Twist (file twist.xvg). The average helical angle per residue is<br>calculated. For alpha helix it is 100 degrees, for 3-10 helices it will be<br>smaller, for 5-helices it will be larger.<br>3. Rise per residue (file rise.xvg). The helical rise per residue is plotted<br>as the difference in Z-coordinate between Ca atoms. For an ideal helix this<br>is 0.15 nm<br>4. Total helix length (file len-ahx.xvg). The total length of the helix in<br>nm. This is simply the average rise (see above) times the number of helical<br>residues (see below).<br>5. Number of helical residues (file n-ahx.xvg). The title says it all.<br>6. Helix Dipole, backbone only (file dip-ahx.xvg).<br>7. RMS deviation from ideal helix, calculated for the Calpha atoms only (file<br>rms-ahx.xvg).<br>8. Average Calpha-Calpha dihedral angle (file phi-ahx.xvg).<br>9. Average Phi and Psi angles (file phipsi.xvg).<br>10. Ellipticity at 222 nm according to Hirst and Brooks</html>";
	String[] flags=new String[]{"-s","-n","-f","-to","-cz","-co"};
	String[] labelText=new String[]{"Input tpr file","Input ndx file","Input xtc file", "*Output g87 file","Output gro file","Output gro file"};
	String[] toolTips=new String[]{"Option: -s ; Filename: topol.tpr ; Type: Input ; Description: Generic run input: tpr tpb tpa xml","Option: -n ; Filename: index.ndx ; Type: Input ; Description: Index file","Option: -f ; Filename: traj.xtc ; Type: Input ; Description: Generic trajectory: xtc trr trj gro g96 pdb","Option: -to ; Filename: gtraj.g87 ; Type: Output, Opt. ; Description: Gromos-87 ASCII trajectory format","Option: -cz ; Filename: zconf.gro ; Type: Output ; Description: Generic structure: gro g96 pdb xml","Option: -co ; Filename: waver.gro ; Type: Output ; Description: Generic structure: gro g96 pdb xml"};
	String[] optFlags=new String[]{"-[no]h","-[no]X","-nice","-[no]xvgr","-[no]free","-[no]rms","-cutoff"};
//	String[] optText=new String[]{"
	JLabel[] labels;
	JButton[] buttons;
	JPanel[] panels;
	JPanel[] reqPanes;
	JPanel[] optionPanes;
	JTextField[] textboxes;
	JCheckBox[] checkboxes;
	JFileChooser jfc=new JFileChooser();
	JTextField tbRun=new JTextField(70);
	JLabel helpLabel=new JLabel(help);
	JLabel titleLabel=new JLabel(title);
	InputStreamHandler ish=new InputStreamHandler();
	Font f=new Font("Serif",Font.BOLD,20);
	int x=0,y=1;
	public g_helix(Container content,String runDir)
	{
		buttons=new JButton[flags.length];
		labels=new JLabel[flags.length];
		textboxes=new JTextField[flags.length];
		checkboxes=new JCheckBox[flags.length];
		panels=new JPanel[flags.length];
		optionPanes=new JPanel[flags.length];
		reqPanes=new JPanel[flags.length];
		
		GridBagLayout gb=new GridBagLayout();
		GridBagConstraints gc=new GridBagConstraints();
		JPanel pane=new JPanel(gb);
		content.add(pane,BorderLayout.CENTER);
		content.add(new JScrollPane(pane));
		titleLabel.setFont(f);
		gc.gridx=0;
		gc.gridy=0;
		gc.weightx=0.2;
		gc.weighty=0.3;
		gb.setConstraints(titleLabel,gc);
		pane.add(titleLabel);
		
		gc.gridx=0;
		gc.gridy=1;
		gb.setConstraints(helpLabel,gc);
		pane.add(helpLabel);
		
		GridBagLayout cgb=new GridBagLayout();
		GridBagConstraints cgc=new GridBagConstraints();
		JPanel centerPane=new JPanel(cgb);
		gc.gridx=0;
		gc.gridy=2;
		gb.setConstraints(centerPane,gc);
		pane.add(centerPane);
		
		GridBagLayout rgb=new GridBagLayout();
		GridBagConstraints rgc=new GridBagConstraints();
		JPanel reqPane=new JPanel(rgb);
		JScrollPane jsp=new JScrollPane(reqPane);
		jsp.setAutoscrolls(true);
		cgc.gridx=0;
		cgc.gridy=0;
		cgb.setConstraints(jsp,cgc);
		centerPane.add(jsp);
		centerPane.setBorder(BorderFactory.createCompoundBorder(BorderFactory.createTitledBorder("Required Parameters"),BorderFactory.createEmptyBorder(10,10,10,10)));
		
		JButton bOk=new JButton("OK");
		bOk.addActionListener(this);
		bOk.setActionCommand("ok");
		cgc.gridx=0;
		cgc.gridy=1;
		cgc.gridwidth=2;
		cgc.gridheight=2;
		cgc.anchor=GridBagConstraints.LINE_END;
		cgc.insets=new Insets(5,5,0,0);
		cgb.setConstraints(bOk,cgc);
		centerPane.add(bOk);
		
		for(int i=0;i<flags.length;i++)
		{
			if(i%2==0)
			{
				y++;
				x=0;
			}
			else
			{
				x=2;
			}				
			labels[i]=new JLabel(labelText[i]);
			panels[i]=new JPanel();
			reqPanes[i]=new JPanel();
			buttons[i]=new JButton("Browse");
			buttons[i].addActionListener(this);
			buttons[i].setActionCommand(flags[i]);
			checkboxes[i]=new JCheckBox(flags[i]);
			checkboxes[i].setToolTipText(toolTips[i]);
			textboxes[i]=new JTextField(10);
			panels[i].add(checkboxes[i]);
			panels[i].add(labels[i]);
			reqPanes[i].add(textboxes[i]);
			reqPanes[i].add(buttons[i]);
			rgc.gridx=x;
			rgc.gridy=y;
			rgc.anchor=GridBagConstraints.LINE_START;
			rgb.setConstraints(panels[i],rgc);
			reqPane.add(panels[i]);
			rgc.gridx=x+1;
			rgc.gridy=y;
			rgc.anchor=GridBagConstraints.LINE_START;
			rgb.setConstraints(reqPanes[i],rgc);
			reqPane.add(reqPanes[i]);
		}
		JPanel runPane=new JPanel(new FlowLayout());
		JButton bRun=new JButton("Run");
		tbRun.setText("g_helix ");
		bRun.addActionListener(this);
		bRun.setActionCommand("run");
		runPane.add(tbRun);
		runPane.add(bRun);
		gc.gridx=0;
		gc.gridy=3;
		gb.setConstraints(runPane,gc);
		pane.add(runPane);
	}
	public void actionPerformed(ActionEvent ae)
	{
		String btn=ae.getActionCommand();
		
		if(ae.getActionCommand().equals("ok"))
		{
			String command="g_helix ";
			for(int i=0;i<flags.length;i++)
			{
				if(checkboxes[i].isSelected())
				{
					command=command+flags[i]+" "+textboxes[i].getText()+" ";
				}
			}
			tbRun.setText(command);
		}
		if(ae.getActionCommand().equals("run"))
		{
			String comm=tbRun.getText();
			try
			{
				Process myProc=Runtime.getRuntime().exec(comm);
				InputStream iStream=myProc.getInputStream();
				InputStream eStream=myProc.getErrorStream();
				ish.handleStream(iStream,eStream);
				int evl=myProc.waitFor();
			}
			catch(IOException ioe)
			{
				System.out.println("IOException\n"+ioe);
			}
			catch(InterruptedException inte)
			{
				System.out.println("InterruptedException\n"+inte);
			}
		}
		for(int i=0;i<flags.length;i++)
		{
			if(btn.equals(flags[i]))
			{
				int retval=jfc.showOpenDialog(null);
				String fname="";
				if(retval==JFileChooser.APPROVE_OPTION)
				{
					File myFile=jfc.getSelectedFile();
					fname=myFile.toString();
					textboxes[i].setText(fname);
				}
			}
		}
	
	}
}
