package vault;
import java.io.*;
import java.awt.*;
import javax.swing.*;
import java.awt.event.*;
import javax.swing.event.*;

public class g_mindist implements ActionListener
{
	String title="g_mindist";
	String help="<html>g_mindist computes the distance between one group and a number of other<br>groups. Both the minimum distance (between any pair of atoms from the<br>respective groups) and the number of contacts within a given distance are<br>written to two separate output files.<br><br>Refer to GUIMACS Help for a detailed description on g_mindist.</html>";
	String[] flags=new String[]{"-f","-s","-n","-od","-on","-o","-ox","-or"};
	String[] labelText=new String[]{"Input xtc file","*Input tpr file","*Input ndx file", "Output xvg file","*Output xvg file","*Output out file","*Output xtc file","*Output xvg file","*Output xvg file"};
	String[] toolTips=new String[]{"Option: -f ; Filename: traj.xtc ; Type: Input ; Description: Generic trajectory: xtc trr trj gro g96 pdb","Option: -s ; Filename: topol.tpr ; Type: Input, Opt. ; Description: Structure+mass(db): tpr tpb tpa gro g96 pdb xml","Option: -n ; Filename: index.ndx ; Type: Input, Opt. ; Description: Index file","Option: -od ; Filename: mindist.xvg ; Type: Output ; Description: xvgr/xmgr file","Option: -on ; Filename: numcont.xvg ; Type: Output, Opt. ; Description: xvgr/xmgr file","Option: -o ; Filename: atm-pair.out ; Type: Output, Opt. ; Description: Generic output file","Option: -ox ; Filename: mindist.xtc ; Type: Output, Opt. ; Description: Generic trajectory: xtc trr trj gro g96 pdb","Option: -or ; Filename: mindistres.xvg ; Type: Output, Opt. ; Description: xvgr/xmgr file"};
	String[] optFlags=new String[]{"-[no]h","-[no]X","-nice","-[no]xvgr","-[no]free","-[no]rms","-cutoff"};
//	String[] optText=new String[]{"
	output out;
	JLabel[] labels;
	JButton[] buttons;
	JPanel[] panels,reqPanes,optionPanes;
	JTextField[] textboxes;
	JCheckBox[] checkboxes;
	JFileChooser jfc=new JFileChooser();
	JButton output=new JButton("View Output");
	JButton bLog,bClg,bCbk,bOut;
	JTextArea tbRun=new JTextArea(5,60);
	JLabel helpLabel=new JLabel(help);
	JLabel titleLabel=new JLabel(title);
	InputStreamHandler ish=new InputStreamHandler();
	Font f=new Font("Serif",Font.BOLD,20);
	JFrame opf;
	int x=0,y=1;
	String comm=null,pwd=null;
	public g_mindist(Container content,String runDir)
	{
		pwd=runDir;
		jfc.setCurrentDirectory(new java.io.File(pwd));
		tbRun.setLineWrap(true);
		tbRun.setWrapStyleWord(true);
		buttons=new JButton[flags.length];
		labels=new JLabel[flags.length];
		textboxes=new JTextField[flags.length];
		checkboxes=new JCheckBox[flags.length];
		panels=new JPanel[flags.length];
		optionPanes=new JPanel[flags.length];
		reqPanes=new JPanel[flags.length];
		
		GridBagLayout gb=new GridBagLayout();
		GridBagConstraints gc=new GridBagConstraints();
		JPanel pane=new JPanel(gb);
		content.add(pane,BorderLayout.CENTER);
		content.add(new JScrollPane(pane));
		titleLabel.setFont(f);
		gc.gridx=0;
		gc.gridy=0;
		gc.weightx=0.2;
		gc.weighty=0.3;
		gb.setConstraints(titleLabel,gc);
		pane.add(titleLabel);
		
		gc.gridx=0;
		gc.gridy=1;
		gb.setConstraints(helpLabel,gc);
		pane.add(helpLabel);
		
		GridBagLayout cgb=new GridBagLayout();
		GridBagConstraints cgc=new GridBagConstraints();
		JPanel centerPane=new JPanel(cgb);
		gc.gridx=0;
		gc.gridy=2;
		gb.setConstraints(centerPane,gc);
		pane.add(centerPane);
		
		GridBagLayout rgb=new GridBagLayout();
		GridBagConstraints rgc=new GridBagConstraints();
		JPanel reqPane=new JPanel(rgb);
		JScrollPane jsp=new JScrollPane(reqPane);
		jsp.setAutoscrolls(true);
		cgc.gridx=0;
		cgc.gridy=0;
		cgb.setConstraints(jsp,cgc);
		centerPane.add(jsp);
		centerPane.setBorder(BorderFactory.createCompoundBorder(BorderFactory.createTitledBorder("Required Parameters"),BorderFactory.createEmptyBorder(10,10,10,10)));
		
		JButton bOk=new JButton("OK");
		bOk.addActionListener(this);
		bOk.setActionCommand("ok");
		cgc.gridx=0;
		cgc.gridy=1;
		cgc.gridwidth=2;
		cgc.gridheight=2;
		cgc.anchor=GridBagConstraints.LINE_END;
		cgc.insets=new Insets(5,5,0,0);
		cgb.setConstraints(bOk,cgc);
		centerPane.add(bOk);
		
		for(int i=0;i<flags.length;i++)
		{
			if(i%2==0)
			{
				y++;
				x=0;
			}
			else
			{
				x=2;
			}				
			labels[i]=new JLabel(labelText[i]);
			panels[i]=new JPanel();
			reqPanes[i]=new JPanel();
			buttons[i]=new JButton("Browse");
			buttons[i].addActionListener(this);
			buttons[i].setActionCommand(flags[i]);
			checkboxes[i]=new JCheckBox(flags[i]);
			checkboxes[i].setToolTipText(toolTips[i]);
			textboxes[i]=new JTextField(10);
			panels[i].add(checkboxes[i]);
			panels[i].add(labels[i]);
			reqPanes[i].add(textboxes[i]);
			reqPanes[i].add(buttons[i]);
			rgc.gridx=x;
			rgc.gridy=y;
			rgc.anchor=GridBagConstraints.LINE_START;
			rgb.setConstraints(panels[i],rgc);
			reqPane.add(panels[i]);
			rgc.gridx=x+1;
			rgc.gridy=y;
			rgc.anchor=GridBagConstraints.LINE_START;
			rgb.setConstraints(reqPanes[i],rgc);
			reqPane.add(reqPanes[i]);
		}
		JPanel runPane=new JPanel(new FlowLayout());
		JButton bRun=new JButton("Run");
		tbRun.setText("g_mindist ");
		bRun.addActionListener(this);
		bRun.setActionCommand("run");
		JScrollPane runsp=new JScrollPane(tbRun,JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		runPane.add(runsp);
		runPane.add(bRun);
		gc.gridx=0;
		gc.gridy=3;
		gb.setConstraints(runPane,gc);
		pane.add(runPane);
		
		gc.gridx=0;
		gc.gridy=4;
		gb.setConstraints(output,gc);
		pane.add(output);
		output.setEnabled(false);
		output.addActionListener(this);
		output.setActionCommand("output");
	}
	public void actionPerformed(ActionEvent ae)
	{
		String btn=ae.getActionCommand();
		
		if(btn.equals("ok"))
		{
			String command="g_mindist ";
			for(int i=0;i<flags.length;i++)
			{
				if(checkboxes[i].isSelected())
				{
					command=command+flags[i]+" "+textboxes[i].getText()+" ";
				}
			}
			tbRun.setText(command);
			command=title+" ";
		}
		if(btn.equals("run"))
		{
			comm=tbRun.getText().trim();
			if(comm.length()==0)
			{
				JOptionPane.showMessageDialog(null,"No run string found.\nPlease enter the run string manually\nor enter values in appropriate boxes above and click \"OK\"","Error",JOptionPane.ERROR_MESSAGE);
				tbRun.requestFocus();
			}
			else
			{
				Thread runner=new Thread()
				{
					public void run()
					{
						try
						{
							BufferedWriter bw=new BufferedWriter(new FileWriter(pwd+"/gmds.exp"));
							bw.write("#!/bin/expect\n");
							bw.write("spawn -noecho "+comm+"\n");
							bw.write("set timeout -1\n");
							bw.write("expect EOF\n");
							bw.close();
							String[] runcomm={"/bin/sh","-c","expect gmds.exp >gmds.log"};
							Process myProc=Runtime.getRuntime().exec(runcomm,null,(new File(pwd)));
							JOptionPane.showMessageDialog(null,"g_mindist started successfully.\nYou will be intimated when the process finishes.","Started...",JOptionPane.INFORMATION_MESSAGE);
							int evl=myProc.waitFor();
							(new File(pwd+"/gmds.exp")).delete();
							JOptionPane.showMessageDialog(null,"GUIMACS finished running g_mindist.\nHowever this may not mean you got the results you are looking for.\nPlease look at the log files to make sure that you got the required results.\n\n"+pwd+"/gmds.log - output log","Done",JOptionPane.INFORMATION_MESSAGE);
							output.setEnabled(true);
						}
						catch(IOException ioe)
						{
							JOptionPane.showMessageDialog(null,ioe,"IOException",JOptionPane.ERROR_MESSAGE);
						}
						catch(InterruptedException inte)
						{
							JOptionPane.showMessageDialog(null,inte,"Interrupted Exception",JOptionPane.ERROR_MESSAGE);
						}
					}
				};
				runner.start();
			}
		}
		if(btn.equals("output"))
		{
			opf=new JFrame("Options");
			opf.setVisible(true);
			opf.setResizable(false);
			opf.setLocationRelativeTo(null);
			Box box=Box.createVerticalBox();
			opf.getContentPane().add(box);
			box.add(Box.createRigidArea(new Dimension(0,20)));
			JPanel pView=new JPanel(new GridLayout(3,3));
			bLog=new JButton("View Output Log File");
			bClg=new JButton("Clean up Log Files");
			bCbk=new JButton("Remove backed up files");
			bOut=new JButton("Close This Window");
			bLog.addActionListener(this);
			bLog.setActionCommand("log");
			bClg.addActionListener(this);
			bClg.setActionCommand("clog");
			bCbk.addActionListener(this);
			bCbk.setActionCommand("cbk");
			bOut.addActionListener(this);
			bOut.setActionCommand("out");
			pView.add(bLog);
			if(!(new File(pwd+"/gmds.log")).exists())
			{
				bLog.setEnabled(false);
			}
			pView.add(bClg);
			if(!(new File(pwd+"/gmds.log")).exists())
			{
				bClg.setEnabled(false);
			}
			pView.add(bCbk);
			pView.add(bOut);
			box.add(pView);
			box.add(Box.createRigidArea(new Dimension(0,20)));
			opf.pack();
		}
		if(btn.equals("log"))
		{
			String filename=pwd+"/gmds.log";
			out=new output(filename);
			out.show();
		}
		if(btn.equals("clog"))
		{
			(new File(pwd+"/gmds.log")).delete();
			JOptionPane.showMessageDialog(null,"Operation Successful");
			bClg.setEnabled(false);
			bLog.setEnabled(false);
		}
		if(btn.equals("cbk"))
		{
			try{
				String[] runcomm={"/bin/sh","-c","rm -f "+pwd+"/\\#*"};
				Process proc=Runtime.getRuntime().exec(runcomm);
				proc.waitFor();
				JOptionPane.showMessageDialog(null,"Cleaned up back up files");
				bCbk.setEnabled(false);
			}
			catch(IOException ioe)
			{
				JOptionPane.showMessageDialog(null,ioe);
			}
			catch(InterruptedException inte)
			{
				JOptionPane.showMessageDialog(null,inte);
			}
		}
		if(btn.equals("out"))
		{
			opf.hide();
		}
		for(int i=0;i<flags.length;i++)
		{
			if(btn.equals(flags[i]))
			{
				int retval=jfc.showOpenDialog(null);
				String fname="";
				if(retval==JFileChooser.APPROVE_OPTION)
				{
					File myFile=jfc.getSelectedFile();
					fname=myFile.toString();
					textboxes[i].setText(fname);
				}
			}
		}
	
	}
}
